poseidon library

Constants

poseidonNumToHashFN → const List<BigInt Function(List<BigInt> inputs)>
Poseidon hash functions indexed by input arity, where poseidonNumToHashFN[n - 1] hashes a list of n field elements.

Functions

poseidonHash(List<BigInt> inputs) BigInt
Computes the Poseidon hash of inputs over the BN254 scalar field.